#define _SVID_SOURCE
#define _GNU_SOURCE
#define _POSIX_SOURCE

#include <arpa/inet.h>
#include "main.h"
#include "fils.h"

config_outil_IPC_t * conf_outil_IPC = NULL;
void *memoire_partager_pere = NULL;
// Permet de gerer les signaux en raport avec le logger
void gestionaire_signaux(int num_signal)
{
  if(num_signal == SIGQUIT || num_signal == SIGINT)
  {
    if(conf_outil_IPC != NULL)
    {
      msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
      shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
    }
    if(memoire_partager_pere != NULL)
    {
      int i;
      memoire_switch_t * memoire_switch = (memoire_switch_t *) memoire_partager_pere;
      for(i = 0; i < memoire_switch->nbPort; i++)
      {
	if(memoire_switch->Port[i].pid != 0)
	{
	  kill(memoire_switch->Port[i].pid,SIGQUIT);
	}
      }
    }
     // on declanche la destruction de tout
     exit(EXIT_SUCCESS);
  }
}
  


int main(int argc, char* argv[])
{
  if(argc != 3)
  {
      printf("./virtualSwitch 2048 8\n");
      exit(-1);
  }
  
  conf_outil_IPC = malloc(sizeof(config_outil_IPC_t));
  
  int nbPort = atoi(argv[2]);

  
  
  // definition du signal
  struct sigaction ordre;
  sigemptyset(&(ordre.sa_mask));
  
  ordre.sa_handler = gestionaire_signaux;
  ordre.sa_flags = 0;
  
  sigaction(SIGQUIT,&ordre,NULL);
  sigaction(SIGINT,&ordre,NULL);

  
  
  conf_outil_IPC->clefFM = FILE_MESSAGE_ID;
  conf_outil_IPC->clefMP = SEG_MEMOIRE_ID;
  
  //creation de la file de message
  conf_outil_IPC->file_message_id = -1;
  conf_outil_IPC->seg_memoire_id = -1;
  while(conf_outil_IPC->file_message_id == -1)
  {
      if((conf_outil_IPC->file_message_id = msgget(conf_outil_IPC->clefFM, S_IRUSR | S_IWUSR|  IPC_CREAT | IPC_EXCL)) == -1)
      {
	if(errno == EEXIST)
	{
	  conf_outil_IPC->clefFM += 1;
	}
	else
	{
	  printf("Une erreur est survenue lors de la creation de la file de message\n");
	  exit(EXIT_FAILURE);
	}
	
      }
  }

  
 
  
  
  
  //configuration du segment de memoire 
  while(conf_outil_IPC->seg_memoire_id == -1)
  {
    if((conf_outil_IPC->seg_memoire_id = shmget(conf_outil_IPC->clefMP, (nbPort*sizeof(Port_t))+sizeof(memoire_switch_t) ,S_IRUSR | S_IWUSR|  IPC_CREAT | IPC_EXCL)) == -1)
    {
      if(errno == EEXIST)
      {
	conf_outil_IPC->clefMP += 1;
      }
      else
      {
	printf("Une erreur est survenue lors de la creation de la memoire partager\n");
	msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
	exit(EXIT_FAILURE);
      }
      
    }
  }
  
  
  //attachement du segment de memoire partager
  if((memoire_partager_pere = shmat(conf_outil_IPC->seg_memoire_id, NULL,0)) == (void *)-1)
  {
    printf("Une erreur est survenue lors de l'attachement de la memoire partager\n");
    msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
    shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
    exit(EXIT_FAILURE);
  }
  
  //creation d'un pointeur vers la memoire partager et mise a 0 de toute la meloire
  memoire_switch_t * memoire_switch = (memoire_switch_t *) memoire_partager_pere;
  memset(memoire_partager_pere,0,(nbPort*sizeof(Port_t))+sizeof(memoire_switch_t));
  
  //intitialisation
  memoire_switch->nbPort = nbPort;
  memoire_switch->Port = (Port_t * )(memoire_partager_pere + sizeof(memoire_switch_t));
  
  
  
  int port_config = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if(port_config == -1)
  {
      fprintf(stderr, "Erreur lors de la creation de la socket pour le port de config : %s\n",strerror(errno));
      msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
      shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
      exit(EXIT_FAILURE);
  }
  
  
  struct sockaddr_in localaddr ;
  memset(&localaddr,0,sizeof(struct sockaddr_in));
  localaddr.sin_family = AF_INET; 
  localaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  localaddr.sin_port = htons(atoi(argv[1]));
  
  
  if(bind(port_config,(struct sockaddr*)&localaddr,sizeof(struct sockaddr_in)) == -1)
  {
    fprintf(stderr, "Erreur lors du Bind pour le port de config : %s\n",strerror(errno));
    msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
    shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
    close(port_config);
    exit(EXIT_FAILURE);
  }
  
  
 
  while(1)
  {
    sleep(1);
    receptionnerCommande(port_config,memoire_switch);
  }
  
  

  
}

int creePortswitch(memoire_switch_t * memoire_switch,int trunk,char* IP_Dest,int client)
{
  ////////////////VERIFICATION DE DISPONIBILISTER DES PORT
  
  int i;
  int portlibre = -1;
  int case_a_utiliser = 0;
  for(i = 0; i < memoire_switch->nbPort; i++)
  {
    if(memoire_switch->Port[i].pid == 0)
    {
      case_a_utiliser = i;
      i = memoire_switch->nbPort;
      portlibre = 1;
      
    }
  }
  if(portlibre == -1)
  {
    return -1;
  }
  
  
  //////////////// CREATION D'UN PORT
  int port = select_port();
  
  //creation d'un tube pour la configuration du port
  int tube[2];
  pipe(tube);
  
  
  if(fork() == 0)
  {
    close(tube[1]);
    fils(port,tube[0],case_a_utiliser,conf_outil_IPC,1,trunk,IP_Dest,client);
    exit(0);
  }
  
  close(tube[0]);
  memoire_switch->Port[case_a_utiliser].tubecommande = tube[1];
  
  return port;
  
}

int select_port()
{
  
    struct sockaddr_in client;         
    int sock = socket(AF_INET, SOCK_STREAM, 0);
    if(sock == -1)
    {
      fprintf(stderr, "Erreur lors de la creation de la socket pour le test de port: %s\n",strerror(errno));
    }
    client.sin_addr.s_addr = htonl(INADDR_ANY);
    client.sin_family = AF_INET;
    
    
    int port = rand()%(30000-20000) + 20000;
    
    
    client.sin_port = htons(port);  
        
    while(connect(sock, (struct sockaddr *) &client,sizeof(client)) != -1)
    {
      close(sock);
      sock = socket(AF_INET, SOCK_STREAM, 0);
      port = rand()%(30000-20000) + 20000;
      client.sin_port = htons(port);
    }
    
    close(sock);
    return port;
  
  
}
int initializeconnectionswitch(struct sockaddr_in localaddr)
{
    int port_config = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(port_config == -1)
    {
	fprintf(stderr, "Erreur lors de la creation de la socket pour le port de config : %s\n",strerror(errno));
	exit(EXIT_FAILURE);
    }
    printf("INTITIALISATION de la conection avec le switch distant\n");
    
    
    Commande_t ma_commande;
    ma_commande.ordre = creationporttrunk;

    // on envoie la commande pour cree un port au switch
    sendto(port_config, &ma_commande, sizeof(Commande_t), 0,(struct sockaddr *)&localaddr, sizeof(struct sockaddr_in));
    
    ReponseCommande_t var;
    
    if(recv(port_config, &var, sizeof(ReponseCommande_t), 0) == -1)
    {
      fprintf(stderr, "Erreur lors de la reception d'ordre pour le port de config : %s\n",strerror(errno));
      return -1;
    }
    close(port_config);
    
    return var.port;
      
    
}
int setVLAN(memoire_switch_t * memoire_switch)
{
  
  return 0;
}
void receptionnerCommande(int socket,memoire_switch_t * memoire_switch)
{
  printf("Attente d'ordre\n");
  
  Commande_t ma_commande;
  struct sockaddr_in adresse_source;
  socklen_t adresse_length = sizeof(struct sockaddr);;
  
  errno = 0;
  if(recvfrom(socket, &ma_commande, sizeof(Commande_t), 0,(struct sockaddr*)&adresse_source, &adresse_length) == -1)
  {
    fprintf(stderr, "Erreur lors de la reception d'ordre pour le port de config : %s\n",strerror(errno));
    return;
  }
 
  int port;
  struct sockaddr_in adresseServeur ;
  memset(&adresseServeur,0,sizeof(struct sockaddr_in));
  adresseServeur = ma_commande.IPdest;
  
  char * var = malloc(sizeof(char)*17);
  inet_ntop(AF_INET, &(ma_commande.IPdest), var, INET_ADDRSTRLEN);
  
  switch(ma_commande.ordre)
  {
    case creeport:
      printf("creation du port en cour\n");
      port = creePortswitch(memoire_switch,0,NULL,0);
      printf("port %d \n",port);
      reponseCommandeCreePort(socket,adresse_source, port );
      break;
    case VLAN:
    
    break;
    case ConnexionInterSwitch:
      // envoie de la commande de creation de port trunck au switch distant 
      port = initializeconnectionswitch(adresseServeur);
      creePortswitch(memoire_switch,1,var,port);
      
    break;
    case creationporttrunk:
      port = creePortswitch(memoire_switch,1,NULL,0);
      reponseCommandeCreePort(socket,adresse_source, port);
      break;
    case getStatue:
    
    break;
  }
  
  
  
  
}
void reponseCommandeCreePort(int socket,struct sockaddr_in adresse_source, int port )
{
  
  ReponseCommande_t var;
  var.statue = 1;
  var.port = port;
  sendto(socket, &var, sizeof(ReponseCommande_t), 0,&adresse_source, sizeof(struct sockaddr_in));
  
}
void reponseCommandeOKFAIL(int socket ,struct sockaddr_in adresse_source,int ok)
{
  ReponseCommande_t var;
  var.statue = ok;
  sendto(socket, &var, sizeof(ReponseCommande_t), 0,&adresse_source, sizeof(struct sockaddr_in));
  
}
void reponseCommandegetStatue(int socket,struct sockaddr_in adresse_source)
{
  /*ReponseCommande_t var;
  var.statue = 1;
  var.port = port;
  sendto(socket, &var, sizeof(ReponseCommande_t), 0,adresse_source, sizeof(struct sockaddr_in));*/
  
}

